@@ -29,7 +29,6 @@ in_source_tree = (
     ANDROID_DIR.name == "Android" and (PYTHON_DIR / "pyconfig.h.in").exists()
 )
 
-ENV_SCRIPT = ANDROID_DIR / "android-env.sh"
 TESTBED_DIR = ANDROID_DIR / "testbed"
 CROSS_BUILD_DIR = PYTHON_DIR / "cross-build"
 
@@ -130,11 +129,12 @@ def android_env(host):
         sysconfig_filename = next(sysconfig_files).name
         host = re.fullmatch(r"_sysconfigdata__android_(.+).py", sysconfig_filename)[1]
 
+    env_script = ANDROID_DIR / "android-env.sh"
     env_output = subprocess.run(
         f"set -eu; "
         f"HOST={host}; "
         f"PREFIX={prefix}; "
-        f". {ENV_SCRIPT}; "
+        f". {env_script}; "
         f"export",
         check=True, shell=True, capture_output=True, encoding='utf-8',
     ).stdout
@@ -151,7 +151,7 @@ def android_env(host):
                 env[key] = value
 
     if not env:
-        raise ValueError(f"Found no variables in {ENV_SCRIPT.name} output:\n"
+        raise ValueError(f"Found no variables in {env_script.name} output:\n"
                          + env_output)
     return env
 
@@ -195,7 +195,7 @@ def unpack_deps(host, prefix_dir):
     os.chdir(prefix_dir)
     deps_url = "https://github.com/beeware/cpython-android-source-deps/releases/download"
     for name_ver in ["bzip2-1.0.8-3", "libffi-3.4.4-3", "openssl-3.0.18-0",
-                     "sqlite-3.50.4-0", "xz-5.4.6-1"]:
+                     "sqlite-3.50.4-0", "xz-5.4.6-1", "zstd-1.5.7-1"]:
         filename = f"{name_ver}-{host}.tar.gz"
         download(f"{deps_url}/{name_ver}/{filename}")
         shutil.unpack_archive(filename)
@@ -281,30 +281,15 @@ def clean_all(context):
 
 
 def setup_ci():
-    if "GITHUB_ACTIONS" in os.environ:
-        # Enable emulator hardware acceleration
-        # (https://github.blog/changelog/2024-04-02-github-actions-hardware-accelerated-android-virtualization-now-available/).
-        if platform.system() == "Linux":
-            run(
-                ["sudo", "tee", "/etc/udev/rules.d/99-kvm4all.rules"],
-                input='KERNEL=="kvm", GROUP="kvm", MODE="0666", OPTIONS+="static_node=kvm"\n',
-                text=True,
-            )
-            run(["sudo", "udevadm", "control", "--reload-rules"])
-            run(["sudo", "udevadm", "trigger", "--name-match=kvm"])
-
-        # Free up disk space by deleting unused versions of the NDK
-        # (https://github.com/freakboy3742/pyspamsum/pull/108).
-        for line in ENV_SCRIPT.read_text().splitlines():
-            if match := re.fullmatch(r"ndk_version=(.+)", line):
-                ndk_version = match[1]
-                break
-        else:
-            raise ValueError(f"Failed to find NDK version in {ENV_SCRIPT.name}")
-
-        for item in (android_home / "ndk").iterdir():
-            if item.name[0].isdigit() and item.name != ndk_version:
-                delete_glob(item)
+    # https://github.blog/changelog/2024-04-02-github-actions-hardware-accelerated-android-virtualization-now-available/
+    if "GITHUB_ACTIONS" in os.environ and platform.system() == "Linux":
+        run(
+            ["sudo", "tee", "/etc/udev/rules.d/99-kvm4all.rules"],
+            input='KERNEL=="kvm", GROUP="kvm", MODE="0666", OPTIONS+="static_node=kvm"\n',
+            text=True,
+        )
+        run(["sudo", "udevadm", "control", "--reload-rules"])
+        run(["sudo", "udevadm", "trigger", "--name-match=kvm"])
 
 
 def setup_sdk():
