@@ -15,14 +15,14 @@
  *    without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE PROJECT AND CONTRIBUTORS ``AS IS'' AND
- * GAI_ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  * ARE DISCLAIMED.  IN NO EVENT SHALL THE PROJECT OR CONTRIBUTORS BE LIABLE
- * FOR GAI_ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+ * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
  * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON GAI_ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN GAI_ANY WAY
+ * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+ * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  */
@@ -51,7 +51,6 @@
 #include <string.h>
 #include <stdlib.h>
 #include <stddef.h>
-#include <ctype.h>
 #include <unistd.h>
 
 #include "addrinfo.h"
@@ -61,6 +60,9 @@
 # define FAITH
 #endif
 
+#ifdef HAVE_NETDB_H
+#define HAVE_GETADDRINFO 1
+
 #define SUCCESS 0
 #define GAI_ANY 0
 #define YES 1
@@ -136,7 +138,7 @@ static int get_addr(const char *, int, struct addrinfo **,
                         struct addrinfo *, int);
 static int str_isnumber(const char *);
 
-static char *ai_errlist[] = {
+static const char * const ai_errlist[] = {
     "success.",
     "address family for hostname not supported.",       /* EAI_ADDRFAMILY */
     "temporary failure in name resolution.",            /* EAI_AGAIN      */
@@ -198,7 +200,7 @@ if (pai->ai_flags & AI_CANONNAME) {\
 
 #define ERR(err) { error = (err); goto bad; }
 
-char *
+const char *
 gai_strerror(int ecode)
 {
     if (ecode < 0 || ecode > EAI_MAX)
@@ -225,8 +227,9 @@ str_isnumber(const char *p)
 {
     unsigned char *q = (unsigned char *)p;
     while (*q) {
-        if (! isdigit(*q))
+        if (!Py_ISDIGIT(*q)) {
             return NO;
+        }
         q++;
     }
     return YES;
@@ -251,7 +254,7 @@ getaddrinfo(const char*hostname, const char*servname,
     if (firsttime) {
         /* translator hack */
         {
-            char *q = getenv("GAI");
+            const char *q = getenv("GAI");
             if (q && inet_pton(AF_INET6, q, &faith_prefix) == 1)
                 translate = YES;
         }
@@ -339,10 +342,18 @@ getaddrinfo(const char*hostname, const char*servname,
                 pai->ai_socktype = SOCK_DGRAM;
                 pai->ai_protocol = IPPROTO_UDP;
             }
-            port = htons((u_short)atoi(servname));
+            long maybe_port = strtol(servname, NULL, 10);
+            if (maybe_port < 0 || maybe_port > 0xffff) {
+                ERR(EAI_SERVICE);
+            }
+            port = htons((u_short)maybe_port);
         } else {
             struct servent *sp;
-            char *proto;
+            const char *proto;
+
+            if (pai->ai_flags & AI_NUMERICSERV) {
+                ERR(EAI_NONAME);
+            }
 
             proto = NULL;
             switch (pai->ai_socktype) {
@@ -481,13 +492,8 @@ getaddrinfo(const char*hostname, const char*servname,
 }
 
 static int
-get_name(addr, gai_afd, res, numaddr, pai, port0)
-    const char *addr;
-    struct gai_afd *gai_afd;
-    struct addrinfo **res;
-    char *numaddr;
-    struct addrinfo *pai;
-    int port0;
+get_name(const char *addr, struct gai_afd *gai_afd, struct addrinfo **res,
+         char *numaddr, struct addrinfo *pai, int port0)
 {
     u_short port = port0 & 0xffff;
     struct hostent *hp;
@@ -527,12 +533,8 @@ get_name(addr, gai_afd, res, numaddr, pai, port0)
 }
 
 static int
-get_addr(hostname, af, res, pai, port0)
-    const char *hostname;
-    int af;
-    struct addrinfo **res;
-    struct addrinfo *pai;
-    int port0;
+get_addr(const char *hostname, int af, struct addrinfo **res,
+         struct addrinfo *pai, int port0)
 {
     u_short port = port0 & 0xffff;
     struct addrinfo sentinel;
@@ -636,3 +638,5 @@ get_addr(hostname, af, res, pai, port0)
     *res = NULL;
     return error;
 }
+
+#endif // HAVE_NETDB_H
